@@ -36,14 +36,22 @@
 # define git_libgit2_init     git_threads_init
 # define git_libgit2_shutdown git_threads_shutdown
 #endif
+#if ! defined (LIBGIT2_SOVERSION) || LIBGIT2_SOVERSION < 23
+/* 0.23 added @p binary_cb */
+# define git_diff_buffers(old_buffer, old_len, old_as_path, \
+                          new_buffer, new_len, new_as_path, options, \
+                          file_cb, binary_cb, hunk_cb, line_cb, payload) \
+  git_diff_buffers (old_buffer, old_len, old_as_path, \
+                    new_buffer, new_len, new_as_path, options, \
+                    file_cb, hunk_cb, line_cb, payload)
+#endif
 
 
 GeanyPlugin      *geany_plugin;
 GeanyData        *geany_data;
-GeanyFunctions   *geany_functions;
 
 
-PLUGIN_VERSION_CHECK(219) /* for document IDs */
+PLUGIN_VERSION_CHECK(224)
 
 PLUGIN_SET_TRANSLATABLE_INFO (
   LOCALEDIR, GETTEXT_PACKAGE,
@@ -411,20 +419,24 @@ worker_thread (gpointer data)
         }
       }
     }
-    if (! repo && git_repository_open_ext (&repo, path, 0, NULL) == 0) {
-      if (git_repository_is_bare (repo)) {
-        git_repository_free (repo);
-        repo = NULL;
-      } else if (G_monitoring_enabled) {
-        /* we need to monitor HEAD, in case of e.g. branch switch (e.g.
-         * git checkout -b will switch the ref we need to watch) */
-        monitors[0] = monitor_repo_file (repo, "HEAD",
-                                         G_CALLBACK (on_git_repo_changed),
-                                         GINT_TO_POINTER (TRUE));
-        /* and of course the real ref (branch) for when changes get committed */
-        monitors[1] = monitor_head_ref (repo, G_CALLBACK (on_git_repo_changed),
-                                        GINT_TO_POINTER (FALSE));
+    if (! repo) {
+      gchar *dirname = g_path_get_dirname (path);
+      if (git_repository_open_ext (&repo, dirname, 0, NULL) == 0) {
+        if (git_repository_is_bare (repo)) {
+          git_repository_free (repo);
+          repo = NULL;
+        } else if (G_monitoring_enabled) {
+          /* we need to monitor HEAD, in case of e.g. branch switch (e.g.
+           * git checkout -b will switch the ref we need to watch) */
+          monitors[0] = monitor_repo_file (repo, "HEAD",
+                                           G_CALLBACK (on_git_repo_changed),
+                                           GINT_TO_POINTER (TRUE));
+          /* and of course the real ref (branch) for when changes get committed */
+          monitors[1] = monitor_head_ref (repo, G_CALLBACK (on_git_repo_changed),
+                                          GINT_TO_POINTER (FALSE));
+        }
       }
+      g_free(dirname);
     }
     
     buf_zero (&job->buf);
@@ -590,14 +602,17 @@ encoding_needs_coversion (const gchar *encoding)
  * @brief Converts encoding
  * @param buffer Input and output buffer
  * @param length Input and output buffer length
+ * @param free_buffer whether @p buffer should be freed when replaced
  * @param to Target encoding
  * @param from Source encoding
  * @param error Return location for errors, or @c NULL
- * @returns @c TRUE if the conversion succeeded and the buffer has been
- *          replaced and should be freed, or @c FALSE otherwise.
+ * @returns @c TRUE if @p buffer should be freed, or @c FALSE otherwise.
  * 
  * @warning This function has a very weird API, but it is practical for
  *          how it's used.
+ * @note the only way to tell if the conversion succeeded when @p free_buffer
+ *       is @c TRUE is to compare the output value of @buffer against the
+ *       one it had as an input value.
  * 
  * Converts between encodings (using g_convert()) in-place.
  * 
@@ -607,12 +622,13 @@ encoding_needs_coversion (const gchar *encoding)
  * of memory that should be freed with @c g_free().
  * If the conversion succeeds, it replaces @p buffer and @p length with
  * the converted values and returns @c TRUE.  If it fails, it does not
- * modify the values and returns @c FALSE so that the passed-in
+ * modify the values and returns @p free_buffer so that the passed-in
  * variables can be used as a fallback if the conversion was optional.
  */
 static gboolean
 convert_encoding_inplace (gchar       **buffer,
                           gsize        *length,
+                          gboolean      free_buffer,
                           const gchar  *to,
                           const gchar  *from,
                           GError      **error)
@@ -622,11 +638,38 @@ convert_encoding_inplace (gchar       **buffer,
                                NULL, &tmp_len, error);
   
   if (tmp_buf) {
+    if (free_buffer) {
+      g_free (*buffer);
+    }
+    
     *buffer = tmp_buf;
     *length = tmp_len;
+    free_buffer = TRUE;
   }
   
-  return tmp_buf != NULL;
+  return free_buffer;
+}
+
+static gboolean
+add_utf8_bom (gchar   **buffer,
+              gsize    *length,
+              gboolean  free_buffer)
+{
+  gchar *new_buf = g_malloc (*length + 3);
+  
+  new_buf[0] = (gchar) 0xef;
+  new_buf[1] = (gchar) 0xbb;
+  new_buf[2] = (gchar) 0xbf;
+  memcpy (&new_buf[3], *buffer, *length);
+  
+  if (free_buffer) {
+    g_free (*buffer);
+  }
+  
+  *buffer = new_buf;
+  *length += 3;
+  
+  return TRUE;
 }
 
 static int
@@ -645,10 +688,15 @@ diff_buf_to_doc (const git_buf   *old_buf,
   buf = (gchar *) scintilla_send_message (sci, SCI_GETCHARACTERPOINTER, 0, 0);
   len = sci_get_length (sci);
   
+  /* add the BOM if needed */
+  if (doc->has_bom) {
+    /* UTF-8 BOM, converted below */
+    free_buf = add_utf8_bom (&buf, &len, free_buf);
+  }
   /* convert the buffer back to in-file encoding if necessary */
   if (encoding_needs_coversion (doc->encoding)) {
-    free_buf = convert_encoding_inplace (&buf, &len, doc->encoding, "UTF-8",
-                                         NULL);
+    free_buf = convert_encoding_inplace (&buf, &len, free_buf,
+                                         doc->encoding, "UTF-8", NULL);
   }
   
   /* no context lines, and no need to bother about binary checks */
@@ -656,7 +704,8 @@ diff_buf_to_doc (const git_buf   *old_buf,
   opts.flags = GIT_DIFF_FORCE_TEXT;
   
   ret = git_diff_buffers (old_buf->ptr, old_buf->size, NULL,
-                          buf, len, NULL, &opts, NULL, hunk_cb, NULL, payload);
+                          buf, len, NULL, &opts, NULL, NULL, hunk_cb, NULL,
+                          payload);
   
   if (free_buf) {
     g_free (buf);
@@ -725,8 +774,8 @@ get_widget_for_buf_range (GeanyDocument *doc,
   
   /* convert the buffer to UTF-8 if necessary */
   if (encoding_needs_coversion (doc->encoding)) {
-    free_buf = convert_encoding_inplace (&buf, &buf_len, "UTF-8", doc->encoding,
-                                         NULL);
+    free_buf = convert_encoding_inplace (&buf, &buf_len, free_buf,
+                                         "UTF-8", doc->encoding, NULL);
   }
   
   scintilla_send_message (sci, SCI_ADDTEXT, buf_len, (glong) buf);
@@ -754,7 +803,7 @@ get_widget_for_buf_range (GeanyDocument *doc,
     }
   }
   gtk_widget_set_size_request (GTK_WIDGET (sci),
-                               MIN (width, alloc.width),
+                               MIN (width + 1, alloc.width),
                                MIN (height + 1, alloc.height));
   
   return GTK_WIDGET (sci);
@@ -1343,10 +1392,16 @@ plugin_configure (GtkDialog *dialog)
   GError     *error   = NULL;
   GtkWidget  *base    = NULL;
   GtkBuilder *builder = gtk_builder_new ();
+#ifdef G_OS_WIN32
+  gchar      *prefix  = g_win32_get_package_installation_directory_of_module (NULL);
+#else
+  gchar      *prefix  = NULL;
+#endif
+  gchar      *path    = g_build_filename (prefix ? prefix : "", PLUGINDATADIR,
+                                          "prefs.ui", NULL);
   
   gtk_builder_set_translation_domain (builder, GETTEXT_PACKAGE);
-  if (! gtk_builder_add_from_file (builder, PKGDATADIR"/"PLUGIN"/prefs.ui",
-                                   &error)) {
+  if (! gtk_builder_add_from_file (builder, path, &error)) {
     g_critical (_("Failed to load UI definition, please check your "
                   "installation. The error was: %s"), error->message);
     g_error_free (error);
@@ -1388,6 +1443,8 @@ plugin_configure (GtkDialog *dialog)
                            cw, (GClosureNotify) configure_widgets_free, 0);
   }
   
+  g_free (path);
+  g_free (prefix);
   g_object_unref (builder);
   
   return base;
